/* 
 * $Header: /home/projects/mule/scm/mule/mule/src/test/java/org/mule/test/transformers/encryption/EncryptionTransformerTestCase.java,v 1.4 2005/04/14 06:08:12 rossmason Exp $
 * $Revision: 1.4 $
 * $Date: 2005/04/14 06:08:12 $
 * ------------------------------------------------------------------------------------------------------
 * 
 * Copyright (c) SymphonySoft Limited. All rights reserved.
 * http://www.symphonysoft.com
 * 
 * The software in this package is published under the terms of the BSD
 * style license a copy of which has been included with this distribution in
 * the LICENSE.txt file. 
 *
 */

package org.mule.test.transformers.encryption;

import org.mule.impl.security.PasswordBasedEncryptionStrategy;
import org.mule.tck.AbstractTransformerTestCase;
import org.mule.transformers.encryption.DecryptionTransformer;
import org.mule.transformers.encryption.EncryptionTransformer;
import org.mule.umo.lifecycle.InitialisationException;
import org.mule.umo.security.CryptoFailureException;
import org.mule.umo.transformer.UMOTransformer;

import java.util.Arrays;

/**
 * @author <a href="mailto:ross.mason@symphonysoft.com">Ross Mason</a>
 * @version $Revision: 1.4 $
 */
public class EncryptionTransformerTestCase extends AbstractTransformerTestCase
{
    private PasswordBasedEncryptionStrategy strat;

    protected void setUp() throws Exception
    {
        super.setUp();
        strat = new PasswordBasedEncryptionStrategy();
        strat.setPassword("mule");
        strat.initialise();
    }


    /* (non-Javadoc)
     * @see org.mule.tck.AbstractTransformerTestCase#getResultData()
     */
    public Object getResultData()
    {
        try
        {
            return strat.encrypt(getTestData().toString().getBytes(), null);
        } catch (CryptoFailureException e)
        {
            fail(e.getMessage());
            return null;
        }
    }

    /* (non-Javadoc)
     * @see org.mule.tck.AbstractTransformerTestCase#getTestData()
     */
    public Object getTestData()
    {
        return "the quick brown fox jumped over the lazy dog the quick brown fox jumped over the lazy dog the quick brown fox jumped over the lazy dog";
    }

    /* (non-Javadoc)
     * @see org.mule.tck.AbstractTransformerTestCase#getTransformers()
     */
    public UMOTransformer getTransformer()
    {
        EncryptionTransformer transformer = new EncryptionTransformer();
        transformer.setStrategy(strat);
        try
        {
            transformer.initialise();
        } catch (InitialisationException e)
        {
            fail(e.getMessage());
        }
        return transformer;
    }

    /* (non-Javadoc)
     * @see junit.framework.TestCase#setUp()
     */


    /* (non-Javadoc)
     * @see org.mule.tck.AbstractTransformerTestCase#getRoundTripTransformer()
     */
    public UMOTransformer getRoundTripTransformer()
    {
        DecryptionTransformer transformer = new DecryptionTransformer();
        transformer.setStrategy(strat);
        transformer.setReturnClass(String.class);
        try
        {
            transformer.initialise();
        } catch (InitialisationException e)
        {
            fail(e.getMessage());
        }
        return transformer;
    }

    public boolean compareResults(Object src, Object result) {
        if(src instanceof byte[]) {
            if(src==null && result ==null) return true;
            if(src==null || result ==null) return false;
            return Arrays.equals((byte[])src, (byte[])result);
        } else {
            return super.compareResults(src, result);
        }
    }
}
